@@ -355,7 +355,8 @@ object_instance_new_resolve(JSContext *context,
                           g_type_name(priv->gtype),
                           g_base_info_get_namespace( (GIBaseInfo*) priv->info),
                           g_base_info_get_name( (GIBaseInfo*) priv->info));
-                return JS_FALSE;
+                g_base_info_unref( (GIBaseInfo*) method_info);
+                return JS_TRUE;
             }
 
             gjs_debug(GJS_DEBUG_GOBJECT,
@@ -479,7 +480,7 @@ object_instance_props_to_g_parameters(JSContext   *context,
         if (!gjs_get_string_id(nameval, &name))
             goto free_array_and_fail;
 
-        if (!gjs_object_require_property(context, props, name, &value))
+        if (!gjs_object_require_property(context, props, "property list", name, &value))
             goto free_array_and_fail;
 
         switch (init_g_param_from_property(context, name,
@@ -558,7 +559,15 @@ wrapped_gobj_toggle_notify(gpointer      data,
 
     runtime = data;
 
-    context = gjs_runtime_get_load_context(runtime);
+    /* The JSContext will be gone if runtime is being destroyed.
+     * In that case we effectively already converted to a weak ref without
+     * doing anything since the keep alive will be collected.
+     * Or if !is_last_ref, we do not want to convert to a strong
+     * ref since we want everything collected on runtime destroy.
+     */
+    context = gjs_runtime_peek_load_context(runtime);
+    if (!context)
+        return;
 
     obj = peek_js_obj(context, gobj);
 
@@ -695,6 +704,7 @@ object_instance_constructor(JSContext *context,
             }
 
             priv->gobj = g_object_newv(gtype, n_params, params);
+            free_g_params(params, n_params);
 
             if (G_IS_INITIALLY_UNOWNED(priv->gobj) &&
                 !g_object_is_floating(priv->gobj)) {
@@ -877,13 +887,14 @@ real_connect_func(JSContext *context,
     ObjectInstance *priv;
     GClosure *closure;
     gulong id;
+    guint signal_id;
     const char *signal_name;
+    GQuark signal_detail;
 
     *retval = INT_TO_JSVAL(0);
 
     priv = priv_from_js(context, obj);
-    gjs_debug(GJS_DEBUG_GOBJECT,
-              "connect obj %p priv %p argc %d", obj, priv, argc);
+    gjs_debug_gsignal("connect obj %p priv %p argc %d", obj, priv, argc);
     if (priv == NULL)
         return JS_FALSE; /* wrong class passed in */
     if (priv->gobj == NULL) {
@@ -907,16 +918,26 @@ real_connect_func(JSContext *context,
         return JS_FALSE;
     }
 
-    closure = gjs_closure_new_marshaled(context, JSVAL_TO_OBJECT(argv[1]), "signal callback");
-    if (closure == NULL)
-        return JS_FALSE;
-
     signal_name = gjs_string_get_ascii_checked(context, argv[0]);
     if (signal_name == NULL) {
-        g_closure_sink(closure);
         return JS_FALSE;
     }
 
+    if (!g_signal_parse_name(signal_name,
+                             G_OBJECT_TYPE(priv->gobj),
+                             &signal_id,
+                             &signal_detail,
+                             FALSE)) {
+        gjs_throw(context, "No signal '%s' on object '%s'",
+                     signal_name,
+                     g_type_name(G_OBJECT_TYPE(priv->gobj)));
+        return JS_FALSE;
+    }
+
+    closure = gjs_closure_new_for_signal(context, JSVAL_TO_OBJECT(argv[1]), "signal callback", signal_id);
+    if (closure == NULL)
+        return JS_FALSE;
+
     id = g_signal_connect_closure(priv->gobj,
                                   signal_name,
                                   closure,
@@ -963,8 +984,7 @@ disconnect_func(JSContext *context,
     *retval = JSVAL_VOID;
 
     priv = priv_from_js(context, obj);
-    gjs_debug(GJS_DEBUG_GOBJECT,
-              "disconnect obj %p priv %p argc %d", obj, priv, argc);
+    gjs_debug_gsignal("disconnect obj %p priv %p argc %d", obj, priv, argc);
 
     if (priv == NULL)
         return JS_FALSE; /* wrong class passed in */
@@ -1010,8 +1030,7 @@ emit_func(JSContext *context,
     *retval = JSVAL_VOID;
 
     priv = priv_from_js(context, obj);
-    gjs_debug(GJS_DEBUG_GOBJECT,
-              "emit obj %p priv %p argc %d", obj, priv, argc);
+    gjs_debug_gsignal("emit obj %p priv %p argc %d", obj, priv, argc);
 
     if (priv == NULL)
         return JS_FALSE; /* wrong class passed in */
@@ -1073,11 +1092,13 @@ emit_func(JSContext *context,
         value = &instance_and_args[i + 1];
 
         g_value_init(value, signal_query.param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE);
-        if (!gjs_value_to_g_value(context, argv[i+1],
-                                     value)) {
-            failed = TRUE;
+        if ((signal_query.param_types[i] & G_SIGNAL_TYPE_STATIC_SCOPE) != 0)
+            failed = !gjs_value_to_g_value_no_copy(context, argv[i+1], value);
+        else
+            failed = !gjs_value_to_g_value(context, argv[i+1], value);
+
+        if (failed)
             break;
-        }
     }
 
     if (!failed) {
@@ -1139,6 +1160,44 @@ static JSFunctionSpec gjs_object_instance_proto_funcs[] = {
     { NULL }
 };
 
+static JSBool
+gjs_define_static_methods(JSContext    *context,
+                          JSObject     *constructor,
+                          GIObjectInfo *object_info)
+{
+    int i;
+    const char *obj_name;
+    int n_methods;
+
+    obj_name = g_base_info_get_name ((GIBaseInfo*)object_info);
+    n_methods = g_object_info_get_n_methods(object_info);
+
+    for (i = 0; i < n_methods; i++) {
+        GIFunctionInfo *meth_info;
+        GIFunctionInfoFlags flags;
+        const char *name;
+
+        meth_info = g_object_info_get_method(object_info, i);
+        name = g_base_info_get_name((GIBaseInfo*)meth_info);
+        flags = g_function_info_get_flags (meth_info);
+
+        /* Anything that isn't a method we put on the prototype of the
+         * constructor.  This includes <constructor> introspection
+         * methods, as well as the forthcoming "static methods"
+         * support.  We may want to change this to use
+         * GI_FUNCTION_IS_CONSTRUCTOR and GI_FUNCTION_IS_STATIC or the
+         * like in the near future.
+         */
+        if (!(flags & GI_FUNCTION_IS_METHOD)) {
+            gjs_define_function(context, constructor, meth_info);
+        }
+
+        g_base_info_unref((GIBaseInfo*) meth_info);
+    }
+    return JS_TRUE;
+}
+
+
 JSBool
 gjs_define_object_class(JSContext    *context,
                         JSObject     *in_object,
@@ -1149,6 +1208,7 @@ gjs_define_object_class(JSContext    *context,
 {
     const char *constructor_name;
     JSObject *prototype;
+    JSObject *constructor;
     GIObjectInfo *parent_info;
     JSObject *parent_proto;
     jsval value;
@@ -1229,7 +1289,6 @@ gjs_define_object_class(JSContext    *context,
     }
 
     if (gjs_object_get_property(context, in_object, constructor_name, &value)) {
-        JSObject *constructor;
 
         if (!JSVAL_IS_OBJECT(value)) {
             gjs_throw(context, "Existing property '%s' does not look like a constructor",
@@ -1310,26 +1369,31 @@ gjs_define_object_class(JSContext    *context,
     g_base_info_ref( (GIBaseInfo*) priv->info);
     priv->gtype = gtype;
 
-    gjs_debug(GJS_DEBUG_GOBJECT, "Defined class %s prototype is %p class %p in object %p",
+    gjs_debug(GJS_DEBUG_GOBJECT, "Defined class %s prototype %p class %p in object %p",
               constructor_name, prototype, JS_GetClass(context, prototype), in_object);
 
-    if (constructor_p) {
-        *constructor_p = NULL;
-        gjs_object_get_property(context, in_object, constructor_name, &value);
-        if (value != JSVAL_VOID) {
-            if (!JSVAL_IS_OBJECT(value)) {
-                gjs_throw(context, "Property '%s' does not look like a constructor",
-                          constructor_name);
-                return JS_FALSE;
-            }
+    /* Now get the constructor we defined in
+     * gjs_init_class_dynamic
+     */
+    gjs_object_get_property(context, in_object, constructor_name, &value);
+    constructor = NULL;
+    if (value != JSVAL_VOID) {
+       if (!JSVAL_IS_OBJECT(value)) {
+            gjs_throw(context, "Property '%s' does not look like a constructor",
+                      constructor_name);
+            return JS_FALSE;
+       }
 
-            *constructor_p = JSVAL_TO_OBJECT(value);
-        }
+       constructor = JSVAL_TO_OBJECT(value);
+       gjs_define_static_methods(context, constructor, info);
     }
 
     if (prototype_p)
         *prototype_p = prototype;
 
+    if (constructor_p)
+        *constructor_p = constructor;
+
     return JS_TRUE;
 }
 
@@ -1359,9 +1423,10 @@ get_obj_key(JSRuntime *runtime,
         buf[1] = 's';
         buf[2] = '-';
         d.ptr = runtime;
-        for (i = 0; i < sizeof(void*)*2; i += 2) {
-            buf[OBJ_KEY_PREFIX_LEN+i] = 'a' + ((d.bytes[i] & 0xf0) >> 4);
-            buf[OBJ_KEY_PREFIX_LEN+i+1] = 'a' + (d.bytes[i] & 0x0f);
+        for (i = 0; i < sizeof(void*); i++) {
+        	int offset = OBJ_KEY_PREFIX_LEN+(i*2);
+        	buf[offset] = 'a' + ((d.bytes[i] & 0xf0) >> 4);
+        	buf[offset+1] = 'a' + (d.bytes[i] & 0x0f);
         }
         buf[OBJ_KEY_LEN] = '\0';
         strcpy(cached_buf, buf);
